Index: apc_sma.c
===================================================================
--- apc_sma.c	(revision 306221)
+++ apc_sma.c	(working copy)
@@ -117,7 +117,11 @@
 
 
 /* {{{ MINBLOCKSIZE */
+#ifdef APC_PAGESIZE
+#define MINBLOCKSIZE (APC_PAGESIZE)
+#else
 #define MINBLOCKSIZE (ALIGNWORD(1) + ALIGNWORD(sizeof(block_t)))
+#endif
 /* }}} */
 
 #if 0
@@ -188,6 +192,12 @@
 
     realsize = ALIGNWORD(size + block_size);
 
+#ifdef APC_PAGESIZE
+    if (realsize % APC_PAGESIZE > 0) {
+        realsize += APC_PAGESIZE - (realsize % APC_PAGESIZE);
+    }
+#endif
+
     /*
      * First, insure that the segment contains at least realsize free bytes,
      * even if they are not contiguous.
@@ -381,7 +403,11 @@
         header = (sma_header_t*) shmaddr;
         apc_lck_create(NULL, 0, 1, header->sma_lock);
         header->segsize = sma_segsize;
+#ifdef APC_PAGESIZE
+        header->avail = sma_segsize - APC_PAGESIZE - ALIGNWORD(sizeof(block_t));
+#else
         header->avail = sma_segsize - ALIGNWORD(sizeof(sma_header_t)) - ALIGNWORD(sizeof(block_t)) - ALIGNWORD(sizeof(block_t));
+#endif
 #if ALLOC_DISTRIBUTION
         {
            int j;
@@ -389,16 +415,24 @@
         }
 #endif
         first = BLOCKAT(ALIGNWORD(sizeof(sma_header_t)));
+#ifdef APC_PAGESIZE
+        first->fnext = APC_PAGESIZE;
+#else
+        first->fnext = ALIGNWORD(sizeof(sma_header_t)) + ALIGNWORD(sizeof(block_t));
+#endif
         first->size = 0;
-        first->fnext = ALIGNWORD(sizeof(sma_header_t)) + ALIGNWORD(sizeof(block_t));
         first->fprev = 0;
         first->prev_size = 0;
         SET_CANARY(first);
 #ifdef __APC_SMA_DEBUG__
-        block->id = -1;
+        first->id = -1;
 #endif
         empty = BLOCKAT(first->fnext);
+#ifdef APC_PAGESIZE
+	empty->size = header->avail - APC_PAGESIZE;
+#else
         empty->size = header->avail - ALIGNWORD(sizeof(block_t));
+#endif
         empty->fnext = OFFSET(empty) + empty->size;
         empty->fprev = ALIGNWORD(sizeof(sma_header_t));
         empty->prev_size = 0;
Index: config.m4
===================================================================
--- config.m4	(revision 309115)
+++ config.m4	(working copy)
@@ -226,6 +238,51 @@
   AC_MSG_RESULT(no)
 ])
 
+
+AC_MSG_CHECKING(size of page we should allocate)
+AC_ARG_ENABLE(apc-page,
+[  --enable-apc-page[=ARG] Allocate memory in pages of given size],
+[
+  PHP_APC_PAGE=$enableval
+
+  if test "$PHP_APC_PAGE" == "yes"; then
+	AC_RUN_IFELSE(
+		[
+			#include <stdio.h>
+			#include <unistd.h>
+			int main(void) {
+				int sz;
+				FILE* fp;
+
+				sz = getpagesize();
+
+				fp = fopen("conftest.out", "w");
+				fprintf(fp, "%d", sz);
+				fclose(fp);
+
+				return 0;
+			}
+		],
+		[ 
+		  enableval=`cat conftest.out`
+		  PHP_APC_PAGE=$enableval
+		  AC_DEFINE_UNQUOTED(APC_PAGESIZE, ${enableval}, [ ])
+		  AC_MSG_RESULT($enableval)
+		],
+		[ AC_MSG_FAILURE(could not determine page size) ],
+		[ AC_MSG_FAILURE(cannot automatically determine page size when cross-compiling, please specify manually) ]
+	)
+  else
+	AC_DEFINE_UNQUOTED(APC_PAGESIZE, ${enableval}, [ ])
+	AC_MSG_RESULT($enableval)
+  fi
+],
+[
+  PHP_APC_PAGE=no
+  AC_MSG_RESULT(unpaged)
+])
+
+
 if test "$PHP_APC" != "no"; then
   test "$PHP_APC_MMAP" != "no" && AC_DEFINE(APC_MMAP, 1, [ ])
   test "$PHP_APC_FILEHITS" != "no" && AC_DEFINE(APC_FILEHITS, 1, [ ])
